/*===========================================================================
*
*                            PUBLIC DOMAIN NOTICE
*               National Center for Biotechnology Information
*
*  This software/database is a "United States Government Work" under the
*  terms of the United States Copyright Act.  It was written as part of
*  the author's official duties as a United States Government employee and
*  thus cannot be copyrighted.  This software/database is freely available
*  to the public for use. The National Library of Medicine and the U.S.
*  Government have not placed any restriction on its use or reproduction.
*
*  Although all reasonable efforts have been taken to ensure the accuracy
*  and reliability of the software and data, the NLM and the U.S.
*  Government do not and cannot warrant the performance or results that
*  may be obtained by using this software or data. The NLM and the U.S.
*  Government disclaim all warranties, express or implied, including
*  warranties of performance, merchantability or fitness for any particular
*  purpose.
*
*  Please cite the author in any work or product based on this material.
*
* ===========================================================================
*
*/

#ifndef _h_arch_impl_
#define _h_arch_impl_

#ifndef _h_noarch_arch_impl_
#include "../noarch/arch-impl.h"
#endif

#ifdef __cplusplus
extern "C" {
#endif

typedef __int128 int128_t;

static __inline
int64_t int128_hi ( const int128_t *self )
{
    return ( ( const int64_t* ) self ) [ 1 ];
}

static __inline
uint64_t int128_lo ( const int128_t *self )
{
    return ( ( const uint64_t* ) self ) [ 0 ];
}

static __inline
void int128_sethi ( int128_t *self, int64_t i )
{
    ( ( int64_t* ) self ) [ 1 ] = i;
}

static __inline
void int128_setlo ( int128_t *self, uint64_t i )
{
    ( ( uint64_t* ) self ) [ 0 ] = i;
}

typedef __uint128 uint128_t;

static __inline
uint64_t uint128_hi ( const uint128_t *self )
{
    return ( ( const uint64_t* ) self ) [ 1 ];
}

static __inline
uint64_t uint128_lo ( const uint128_t *self )
{
    return ( ( const uint64_t* ) self ) [ 0 ];
}

static __inline
void uint128_sethi ( uint128_t *self, uint64_t i )
{
    ( ( uint64_t* ) self ) [ 1 ] = i;
}

static __inline
void uint128_setlo ( uint128_t *self, uint64_t i )
{
    ( ( uint64_t* ) self ) [ 0 ] = i;
}

static __inline
void int128_add ( int128_t *self, const int128_t *i )
{
    * self += * i;
}

static __inline
void int128_sub ( int128_t *self, const int128_t *i )
{
    * self -= * i;
}

static __inline
void int128_sar ( int128_t *self, uint32_t i )
{
    * self >>= i;
}

static __inline
void int128_shl ( int128_t *self, uint32_t i )
{
    * self << i;
}

static __inline
void uint128_and ( uint128_t *self, const uint128_t *i )
{
    * self &= * i;
}

static __inline
void uint128_or ( uint128_t *self, const uint128_t *i )
{
    * self |= * i;
}

static __inline
void uint128_orlo ( uint128_t *self, uint64_t i )
{
    ( ( uint64_t* ) self ) [ 0 ] |= i;
}

static __inline
void uint128_xor ( uint128_t *self, const uint128_t *i )
{
    * self ^= * i;
}

static __inline
void uint128_not ( uint128_t *self )
{
    * self = ~ * self;
}

static __inline
void uint128_shr ( uint128_t *self, uint32_t i )
{
    * self >>= i;
}

static __inline
void uint128_shl ( uint128_t *self, uint32_t i )
{
    * self <<= i;
}

static __inline
void uint128_bswap ( uint128_t *self )
{
    uint64_t tmp = _byteswap_uint64 ( ( ( const uint64_t* ) self ) [ 0 ] );
    ( ( uint64_t* ) self ) [ 0 ] = _byteswap_uint64 ( ( ( const uint64_t* ) self ) [ 1 ] );
    ( ( uint64_t* ) self ) [ 1 ] = tmp;
}

static __inline
void uint128_bswap_copy ( uint128_t *to, const uint128_t *from )
{
    ( ( uint64_t* ) to ) [ 0 ] = _byteswap_uint64 ( ( ( const uint64_t* ) from ) [ 1 ] );
    ( ( uint64_t* ) to ) [ 1 ] = _byteswap_uint64 ( ( ( const uint64_t* ) from ) [ 0 ] );
}

#ifdef __cplusplus
}
#endif

#endif /* _h_arch_impl_ */
